#!/bin/bash
#
# Script para monitorizar
#
# chkconfig: 2345 80 80
# description: Monitorizar y alertar
# processname: monitord

##############################################################
# Daemon para monitorizar la maquina y comunicar incidencias #
#                                                            #
# Autor: q3aql <q3aql@duck.com>                              #
# Licencia: GPLv2.0                                          #
# Fecha: 06-08-2022                                          #
##############################################################

# CONFIGURACION
alertaCPU=90 # Uso de CPU al que debe alertar
alertaMemoria=85 # Memoria RAM ocupada a la que debe alertar
serviciosAlerta="jellyfin plexmediaserver/Plex kiwiirc unrealircd sbin/sshd sbin/apache2" # Servicios a monitorizar
discosAlerta="/data/Movies /arcvd/Movies" # Puntos de montaje que se deben comprobar
tamanoLogMaximo=500 # Numero de lineas maximas que tendra el log (Minimo usar 200)
comandoAlerta="echo" # Comando usado para mostrar la salida

# VARIABLES GENERALES
carpetaConfig=/var/monitord
carpetaAlertas=${carpetaConfig}/alertas
logPrincipal=${carpetaConfig}/monitord.log
regAlertaCPU=${carpetaAlertas}/cpu.alert
regAlertaMemoria=${carpetaAlertas}/memoria.alert

# Funcion para comprobar y crear los archivos iniciales
function preparar_configuracion() {
  if [ ! -d ${carpetaConfig} ] ; then
    mkdir -p ${carpetaConfig}
  fi
  if [ ! -d ${carpetaAlertas} ] ; then
    mkdir -p ${carpetaAlertas}
  fi
  listaAlertas="${serviciosAlerta} ${discosAlerta} cpu memoria"
  for alert in ${listaAlertas} ; do
    nombreAlerta=$(echo "${alert}" | sed 's/\//-/g')
    if [ ! -f ${carpetaAlertas}/${nombreAlerta}.alert ] ; then
      echo 0 > "${carpetaAlertas}/${nombreAlerta}.alert"
    fi
  done
}

# Funcion para comprobar permisos de administrador
function comprobar_admin() {
  mkdir -p /etc/root &> /dev/null
  administrador=$?
  if [ ${administrador} -eq 0 ] ; then
    rm -rf /etc/root
  else
    horaActual=$(date "+%Y-%m-%d %H:%M")
    echo "${horaActual} @ Es necesario ser root para iniciar el proceso"
    exit
  fi
}

# Funcion para reducir log
function reducir_log() {
  sizeLog=$(wc -l ${logPrincipal} | cut -d " " -f 1)
  if [ ${sizeLog} -ge ${tamanoLogMaximo} ] ; then
    reducirMaximo=$(expr ${tamanoLogMaximo} - 100)
    tail -${reducirMaximo} ${logPrincipal} > ${carpetaConfig}/monitord-red.log
    cat ${carpetaConfig}/monitord-red.log > ${logPrincipal}
  fi
}

# Funcion para la comprobacion de la CPU
function checkeo_cpu() {
  # Check if sysstat is installed
  if [ -f /usr/bin/sar ] ; then
    porcentageCPU=$(sar 1 5 | grep "Average" | sed 's/^.* //' | cut -d "." -f 1)
    porcentageCPU=$(expr 100 - ${porcentageCPU})    
  else
    porcentageCPU=$(top -b -n1 | grep \%Cpu | awk '{print 100-$8}' | cut -d "." -f 1)
  fi
  if [ ${porcentageCPU} -ge ${alertaCPU} ] ; then
    sleep 2
    porcentageCPU=$(top -b -n1 | grep \%Cpu | awk '{print 100-$8}' | cut -d "." -f 1)
    if [ ${porcentageCPU} -ge ${alertaCPU} ] ; then
      verAlerta=$(cat ${regAlertaCPU})
      if [ ${verAlerta} -eq 0 ] ; then
        horaActual=$(date "+%Y-%m-%d %H:%M")
        echo "${horaActual} @ La CPU esta o supera el ${alertaCPU}% de uso" >> ${logPrincipal}
        ${comandoAlerta} " ${horaActual} @ La CPU esta o supera el ${alertaCPU}% de uso"
        echo 1 > ${regAlertaCPU}
      fi
    fi
  else
    verAlerta=$(cat ${regAlertaCPU})
    if [ ${verAlerta} -eq 1 ] ; then
      horaActual=$(date "+%Y-%m-%d %H:%M")
      echo "${horaActual} @ La CPU ha vuelto a un uso por debajo del ${alertaCPU}%" >> ${logPrincipal}
      ${comandoAlerta} "${horaActual} @ La CPU ha vuelto a un uso por debajo del ${alertaCPU}%"
      echo 0 > ${regAlertaCPU}
    fi
  fi
}

# Funcion para la comprobacion de la memoria
function checkeo_memoria() {
  mem_usada=$(free | awk '/Mem:/ { printf("  %5s/%s \n", $3, $2) }' | tr -s " " | cut -d "/" -f 1)
  mem_total=$(free | awk '/Mem:/ { printf("  %5s/%s \n", $3, $2) }' | tr -s " " | cut -d "/" -f 2)
  mem_reglatres=$(expr ${mem_usada} \* 100)
  mem_porcentage=$(expr ${mem_reglatres} / ${mem_total})
  if [ ${mem_porcentage} -ge ${alertaMemoria} ] ; then
    verAlerta=$(cat ${regAlertaMemoria})
    if [ ${verAlerta} -eq 0 ] ; then
      horaActual=$(date "+%Y-%m-%d %H:%M")
      echo "${horaActual} @ La RAM esta o supera el ${alertaMemoria}% de uso" >> ${logPrincipal}
      ${comandoAlerta} "${horaActual} @ La RAM esta o supera el ${alertaMemoria}% de uso"
      echo 1 > ${regAlertaMemoria}
    fi
  else
    verAlerta=$(cat ${regAlertaMemoria})
    if [ ${verAlerta} -eq 1 ] ; then
      horaActual=$(date "+%Y-%m-%d %H:%M")
      echo "${horaActual} @ La RAM ha vuelto a un uso por debajo del ${alertaMemoria}%" >> ${logPrincipal}
      ${comandoAlerta} "${horaActual} @ La RAM ha vuelto a un uso por debajo del ${alertaMemoria}%"
      echo 0 > ${regAlertaMemoria}
    fi
  fi
}

# Funcion para comprobar los servicios
function checkeo_servicios() {
  for servicio in ${serviciosAlerta} ; do
    nombreAlertaServicio=$(echo "${servicio}" | sed 's/\//-/g')
    procesoServicio=$(ps -ef | grep "${servicio}" | grep -v " grep" | head -1)
    if [ -z "${procesoServicio}" ] ; then
      verAlerta=$(cat ${carpetaAlertas}/${nombreAlertaServicio}.alert)
      if [ ${verAlerta} -eq 0 ] ; then
        horaActual=$(date "+%Y-%m-%d %H:%M")
        echo "${horaActual} @ El servicio ${servicio} esta detenido" >> ${logPrincipal}
        ${comandoAlerta} "${horaActual} @ El servicio ${servicio} esta detenido"
        echo 1 > ${carpetaAlertas}/${nombreAlertaServicio}.alert
      fi
    else
      verAlerta=$(cat ${carpetaAlertas}/${nombreAlertaServicio}.alert)
      if [ ${verAlerta} -eq 1 ] ; then
        horaActual=$(date "+%Y-%m-%d %H:%M")
        echo "${horaActual} @ El servicio ${servicio} esta arrancado de nuevo" >> ${logPrincipal}
        ${comandoAlerta} "${horaActual} @ El servicio ${servicio} esta arrancado de nuevo"
        echo 0 > ${carpetaAlertas}/${nombreAlertaServicio}.alert
      fi
    fi
  done
}

# Funcion para comprobar el montaje de discos
function checkeo_discos() {
  for disco in ${discosAlerta} ; do
    nombreAlertaDisco=$(echo "${disco}" | sed 's/\//-/g')
    if [ ! -d "${disco}" ] ; then
      verAlerta=$(cat ${carpetaAlertas}/${nombreAlertaDisco}.alert)
      if [ ${verAlerta} -eq 0 ] ; then
        horaActual=$(date "+%Y-%m-%d %H:%M")
        echo "${horaActual} @ El disco ${disco} no esta montado" >> ${logPrincipal}
        ${comandoAlerta} "${horaActual} @ El disco ${disco} no esta montado"
        echo 1 > ${carpetaAlertas}/${nombreAlertaDisco}.alert
      fi
    else
      verAlerta=$(cat ${carpetaAlertas}/${nombreAlertaDisco}.alert)
      if [ ${verAlerta} -eq 1 ] ; then
        horaActual=$(date "+%Y-%m-%d %H:%M")
        echo "${horaActual} @ El disco ${disco} vuelve a estar montado" >> ${logPrincipal}
        ${comandoAlerta} "${horaActual} @ El disco ${disco} vuelve a estar montado"
        echo 0 > ${carpetaAlertas}/${nombreAlertaDisco}.alert
      fi
    fi
  done
} 

# Funcion para iniciar el proceso
function iniciar_proceso() {
  horaActual=$(date "+%Y-%m-%d %H:%M")
  echo "${horaActual} @ Iniciando proceso de monitord"
  sleep 60
  echo "${horaActual} @ La maquina $(hostname) acaba de iniciarse"
  ${comandoAlerta} "${horaActual} @ La maquina $(hostname) acaba de iniciarse"
  preparar_configuracion
  monitor=0
  while [ ${monitor} -eq 0 ] ; do
    checkeo_cpu
    checkeo_memoria
    checkeo_servicios
    checkeo_discos
    reducir_log
    sleep 10
  done
}

case ${1} in
  start)
    comprobar_admin
    iniciar_proceso
  ;;
  status)
    status_monitord=$(ps -ef | grep "monitord " | grep -v " grep" | head -1)
    if [ -z ${status_monitord} ] ; then
      horaActual=$(date "+%Y-%m-%d %H:%M")
      echo "${horaActual} @ El proceso de monitord no esta ejecutandose"
    else
      horaActual=$(date "+%Y-%m-%d %H:%M")
      echo "${horaActual} @ El proceso de monitord se esta ejecutando"
    fi
  ;;
  stop)
    comprobar_admin
    horaActual=$(date "+%Y-%m-%d %H:%M")
    echo "${horaActual} @ Deteniendo proceso de monitord"
    killall monitord
  ;;
  log)
    echo ""
    cat "${logPrincipal}"
  ;;
  *)
    echo "# Uso: ${0} {start|status|stop|log}"
  ;;
esac 
